@@ -50,13 +50,7 @@ class ADS:
         self.ss = ac.subspaces.Subspaces()
         self.W1 = None
 
-        self.define_evaluations()
-        self.evaluate_x()
-        self.compute_gradients()
-
-        self.normalise()
-
-    def define_evaluations(self):
+    def define_evaluations_random(self):
         added_points = self.n_pts - self.dataset.n_samples
 
         # adding remaining points uniformely distributed in the design space
@@ -75,6 +69,30 @@ class ADS:
 
         self.x = x
 
+    def define_evaluations_by_corners(self, xc_normed, ret=False):
+        """xc_normed should be the HD corner points in the [-1, 1] range"""
+
+        xc = (xc_normed + 1) * (self.bounds[:, 1] - self.bounds[:, 0]) / 2 + self.bounds[:, 0]
+
+        n_corners = xc.shape[0]  # number of corner points
+        nsp = int(self.n_pts / n_corners)  # number of samples to take in each direction
+
+        x = np.array([]).reshape(0, self.dataset.n_params)
+        for corner in xc:
+            norm = np.linalg.norm(corner)
+            corner /= norm
+            vals = np.linspace(0, norm, nsp+1)
+            vals = np.delete(vals, 0)  # deleting first value since it will be 0
+            for val in vals:
+                x = np.vstack((x, vectorize(corner * val).T))
+
+        # adding dataset samples
+        x = np.vstack((x, np.array(self.dataset.x)))
+        if ret:
+            return x
+        else:
+            self.x = x
+
     def evaluate_x(self):
         print('evaluating points')
         xpred = self.dataset.norm_x(pd.DataFrame(self.x, columns=self.dataset.x.columns))
@@ -93,10 +111,30 @@ class ADS:
         # We need to multiply them by dx/dx_normed
         self.grad_normed = self.gradients * (self.bounds[:, 1] - self.bounds[:, 0]) / 2.0
 
-    def compute(self):
+    def compute(self, to_corners=False):
+
+        self.define_evaluations_random()
+        self.evaluate_x()
+        self.compute_gradients()
+        self.normalise()
+
         self.ss.compute(df=self.grad_normed, nboot=500)
         self.W1 = self.ss.eigenvecs[:, :self.n_dims]
 
+        # corners
+        if to_corners:
+            print ('Calculating corner points')
+            y, xc_normed = self.get_corners()
+            self.define_evaluations_by_corners(xc_normed)
+            print('Infilling corners')
+            self.evaluate_x()
+            print('Calcularing gradients')
+            self.compute_gradients()
+            self.normalise()
+
+            self.ss.compute(df=self.grad_normed, nboot=500)
+            self.W1 = self.ss.eigenvecs[:, :self.n_dims]
+
     def get_corners(self):
         """
         Computes the vertices of the zonotope
@@ -109,6 +147,7 @@ class ADS:
             nzv-by-m matrix that contains the corners of the m-dimensional hypercube
             that map to the zonotope vertices
         """
+        print ('Calculating corners')
         y, x = ac.domains.zonotope_vertices(self.W1)
         return y, x
 
@@ -137,34 +176,72 @@ class ADS:
             ax.set_title('C matrix Cumulative Energy')
 
         elif key == 'zonotope':
-            corners = kwargs.get('corners')
+            show_boundary = kwargs.get('show_boundary')
             show_samples = kwargs.get('show_samples')
+            interp = kwargs.get('interp')
+            show_evaluations = kwargs.get('show_evaluations')
+            levels = kwargs.get('levels')
+            to_corners = kwargs.get('to_corners')
+
+            if show_boundary is None:
+                show_boundary = False
+            if show_samples is None:
+                show_samples = False
+            if show_evaluations is None:
+                show_evaluations = False
+            if to_corners is None:
+                to_corners=False
+            if interp is None:
+                interp = 'linear'
+            if levels is None:
+                levels=10
+
+
             # forcing matrix W1 to be the first 2 eigenvectors of C
             W1 = self.ss.eigenvecs[:, :2]
-            y = self.x_normed.dot(W1)
 
-            u1_i = np.linspace(min(y[:, 0]), max(y[:, 0]), 500)
-            u2_i = np.linspace(min(y[:, 1]), max(y[:, 1]), 500)
+            if to_corners:
+                y, xc_normed = ac.domains.zonotope_vertices(W1)
+                x_eval =  self.define_evaluations_by_corners(xc_normed, ret=True)
+                x_eval_normed = self.dataset.norm_x(pd.DataFrame(x_eval, columns=self.dataset.x.columns))
+                qoi = self.dataset.ret_qoi(self.network.predict(x_eval_normed))
+                y = self.forward_map(x_eval)
+            else:
+                qoi=self.qoi
+                y = self.x_normed.dot(W1)
+
+            u1_i = np.linspace(min(y[:, 0]), max(y[:, 0]), 1500)
+            u2_i = np.linspace(min(y[:, 1]), max(y[:, 1]), 1500)
 
             triang = tri.Triangulation(y[:, 0], y[:, 1])
-            interpolator = tri.LinearTriInterpolator(triang, self.qoi)
+            if interp == 'cubic':
+                interpolator = tri.CubicTriInterpolator(triang, qoi)
+            else:
+                interpolator = tri.LinearTriInterpolator(triang, qoi)
+
             U1, U2 = np.meshgrid(u1_i, u2_i)
             QOI = interpolator(U1, U2)
 
-            plt.contourf(U1, U2, QOI)
+            plt.contourf(U1, U2, QOI, levels)
             ax.set_xlabel('$\mathbf{U_{1}}$')
             ax.set_ylabel('$\mathbf{U_{2}}$')
             plt.colorbar()
             ax.set_title('QoI along $\mathbf{U_{1}}$ and $\mathbf{U_{2}}$')
 
-            if corners:
+            if show_boundary:
                 y, x = ac.domains.zonotope_vertices(W1)
                 u1_sorted, u2_sorted = self.sort_xy(y[:, 0], y[:, 1])
+                u1_sorted = np.append(u1_sorted, u1_sorted[0])
+                u2_sorted = np.append(u2_sorted, u2_sorted[0])
                 ax.plot(u1_sorted, u2_sorted, c='k', lw=1, marker='*', markersize=5)
 
             if show_samples:
-                y = self.forward_map(self.dataset.x)
-                ax.plot(y[:, 0], y[:, 1], 'o', mfc='none', c='r')
+                if not show_evaluations:
+                    y = self.forward_map(self.dataset.x)
+                    ax.scatter(y[:, 0], y[:, 1], c=self.dataset.qoi.values, edgecolors='k')
+            if show_evaluations:
+                y = self.forward_map(self.x)
+                ax.scatter(y[:, 0], y[:, 1], c=self.qoi, edgecolors='k')
 
         if key == 'eigs' or key == 'cumsum':
             ax.grid(color='black', lw=0.25)
